package com.smppw.analysis.domain.dao;

import com.smppw.analysis.domain.entity.FundFrequencyDo;
import com.smppw.analysis.domain.service.SecTypeService;
import com.smppw.analysis.infrastructure.persistence.FundFrequencyDoMapper;
import com.smppw.common.pojo.enums.Frequency;
import com.smppw.constants.SecType;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class FundInformationDao {
    private final SecTypeService secTypeService;
    private final FundFrequencyDoMapper fundFrequencyDoMapper;

    public FundInformationDao(SecTypeService secTypeService, FundFrequencyDoMapper fundFrequencyDoMapper) {
        this.secTypeService = secTypeService;
        this.fundFrequencyDoMapper = fundFrequencyDoMapper;
    }

    public Frequency getNavFrequency(String secId) {
        Frequency frequency;
        String fundType = secTypeService.getFundType(secId);
        if (SecType.PRIVATELY_OFFERED_FUND.equals(fundType) ||
                SecType.PUBLICLY_OFFERED_FUNDS.equals(fundType) ||
                SecType.INDEX_FUND.equals(fundType) ||
                SecType.RONGZHI_INDEX.equals(fundType)) {
            String freq = fundFrequencyDoMapper.getNavFrequencyByFundId(secId);
            if ("天".equals(freq)) {
                frequency = Frequency.Daily;
            } else if ("周".equals(freq)) {
                frequency = Frequency.Weekly;
            } else {
                frequency = Frequency.Monthly;
            }
        } else {
            frequency = Frequency.Daily;
        }
        return frequency;
    }

    public Map<String, Frequency> getNavFrequency(List<String> secIdList) {
        Map<String, Frequency> secFrequencyMap = new HashMap<>();

        Map<String, List<String>> secIdTypeMap = secTypeService.getSecIdTypeMap(secIdList);

        List<String> marketIdList = new ArrayList<>();
        List<String> marketPublicFundIdList = secIdTypeMap.get(SecType.PUBLICLY_OFFERED_FUNDS);
        List<String> marketPrivateFundIdList = secIdTypeMap.get(SecType.PRIVATELY_OFFERED_FUND);
        List<String> marketIndexIdList = secIdTypeMap.get(SecType.INDEX_FUND);
        List<String> marketRongzhiIndexIdList = secIdTypeMap.get(SecType.RONGZHI_INDEX);
        List<String> marketCompanyIdList = secIdTypeMap.get(SecType.COMPANY);
        List<String> marketManagerIdList = secIdTypeMap.get(SecType.MANAGER);
        if (marketPublicFundIdList != null) {
            marketIdList.addAll(marketPublicFundIdList);
        }
        if (marketPrivateFundIdList != null) {
            marketIdList.addAll(marketPrivateFundIdList);
        }
        if (marketIndexIdList != null) {
            marketIdList.addAll(marketIndexIdList);
        }
        if (marketRongzhiIndexIdList != null) {
            marketIdList.addAll(marketRongzhiIndexIdList);
        }
        if (marketCompanyIdList != null) {
            marketIdList.addAll(marketCompanyIdList);
        }
        if (marketManagerIdList != null) {
            marketIdList.addAll(marketManagerIdList);
        }

        if (marketIdList.size() > 0) {
            List<FundFrequencyDo> fundFrequencyDos = fundFrequencyDoMapper.getNavFrequencyByFundIdList(marketIdList);
            for (FundFrequencyDo fundFrequencyDo : fundFrequencyDos) {
                String frequencyStr = fundFrequencyDo.getNavFrequency();
                Frequency frequency = Frequency.Monthly;
                if ("天".equals(frequencyStr)) {
                    frequency = Frequency.Daily;
                } else if ("周".equals(frequencyStr)) {
                    frequency = Frequency.Weekly;
                }
                secFrequencyMap.put(fundFrequencyDo.getFundId(), frequency);
            }
        }

//        List<String> privateIdList = new ArrayList<>();
//        List<String> privateFundIdList = secIdTypeMap.get(SecType.PRIVATE_FUND);
//        List<String> factorIdList = secIdTypeMap.get(SecType.FACTOR);
//        List<String> udfIndexIdList = secIdTypeMap.get(SecType.UDF_INDEX);
//        if (privateFundIdList != null) {
//            privateIdList.addAll(privateFundIdList);
//        }
//        if (factorIdList != null) {
//            privateIdList.addAll(factorIdList);
//        }
//        if (udfIndexIdList != null) {
//            privateIdList.addAll(udfIndexIdList);
//        }
//        if (privateIdList != null && privateIdList.size() > 0) {
//            List<CmFundFrequencyDo> cmFundFrequencyDos = cmFundFrequencyDoMapper.getNavFrequencyByFundIdList(privateIdList);
//            for (CmFundFrequencyDo cmFundFrequencyDo : cmFundFrequencyDos) {
//                String frequencyStr = cmFundFrequencyDo.getNavFrequency();
//                Frequency frequency = Frequency.Monthly;
//                if ("天".equals(frequencyStr)) {
//                    frequency = Frequency.Daily;
//                } else if ("周".equals(frequencyStr)) {
//                    frequency = Frequency.Weekly;
//                }
//                secFrequencyMap.put(cmFundFrequencyDo.getFundId(), frequency);
//            }
//        }

        return secFrequencyMap;
    }
}
