package com.smppw.analysis.domain.service.impl;

import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.infrastructure.persistence.IndexesRzIndexDoMapper;
import com.smppw.analysis.domain.service.SecTypeService;
import com.smppw.common.cache.CaffeineLocalCache;
import com.smppw.common.pojo.enums.strategy.Strategy;
import com.smppw.constants.SecType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Rain
 * @date 2023/3/16 10:03
 * @description
 */
@Service
public class SecTypeServiceImpl implements SecTypeService {
    public static final String HF = "HF";
    public static final String MF = "MF";
    public static final String CF = "CF";
    public static final String FA = "FA";
    public static final String CI = "CI";
    public static final String CO = "CO";
    public static final String PL = "PL";
    public static final String PO = "PO";
    public static final String IN = "IN";
    public static final String AP = "AP";
    private static final Map<String, Integer> INDEX_EXIST = MapUtil.newConcurrentHashMap();
    @Autowired
    private IndexesRzIndexDoMapper indexesRzIndexDoMapper;

    @Override
    public String getFundType(String fundId) {
        if (fundId == null) {
            return null;
        }
        if (fundId.startsWith(HF)) {
            return SecType.PRIVATELY_OFFERED_FUND;
        } else if (fundId.startsWith(MF)) {
            return SecType.PUBLICLY_OFFERED_FUNDS;
        } else if (fundId.startsWith(CF)) {
            return SecType.PRIVATE_FUND;
        } else if (fundId.startsWith(FA)) {
            return SecType.FACTOR;
        } else if (fundId.startsWith(CI)) {
            return SecType.UDF_INDEX;
        } else if (fundId.startsWith(CO)) {
            return SecType.COMPANY;
        } else if (fundId.startsWith(PL)) {
            return SecType.MANAGER;
        } else if (fundId.startsWith(PO)) {
            return SecType.COMBINATION;
        } else if (StrUtil.isNumeric(fundId)) {
            if (Strategy.isStrategy(fundId)) {
                return SecType.STRATEGY;
            }
        } else if (fundId.startsWith(IN)) {
            List<String> thirdIndexes = CaffeineLocalCache.getThirdIndexes();
            if (thirdIndexes != null && thirdIndexes.contains(fundId)) {
                return SecType.THIRD_INDEX_FUND;
            }
            List<String> riskOfFreeIdList = CaffeineLocalCache.getRiskOfFreeId();
            if (riskOfFreeIdList != null && riskOfFreeIdList.contains(fundId)) {
                return SecType.RISK_OF_FREE;
            }
            Integer isExist = INDEX_EXIST.get(fundId);
            if (isExist == null) {
                isExist = indexesRzIndexDoMapper.isExist(fundId);
                INDEX_EXIST.put(fundId, isExist);
            }
            if (isExist.compareTo(0) == 0) {
                return SecType.INDEX_FUND;
            } else {
                return SecType.RONGZHI_INDEX;
            }
        } else if (fundId.startsWith(AP)) {
            return SecType.ADVISORY_POOL_CURVE;
        }
        return null;
    }

    @Override
    public Map<String, List<String>> getSecIdTypeMap(List<String> secIdList) {
        Map<String, List<String>> secIdTypeMap = new HashMap<>(10);
        for (String secId : secIdList) {
            String secIdType = getFundType(secId);
            if (secIdTypeMap.containsKey(secIdType)) {
                List<String> list = secIdTypeMap.get(secIdType);
                list.add(secId);
            } else {
                List<String> list = new ArrayList<>();
                list.add(secId);
                secIdTypeMap.put(secIdType, list);
            }
        }
        return secIdTypeMap;
    }
}
